/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.def.element.BusinessArchive;
import org.ow2.bonita.facade.def.element.ExternalPackageDefinition;
import org.ow2.bonita.facade.def.element.TypeDeclarationDefinition;
import org.ow2.bonita.facade.def.majorElement.ApplicationDefinition;
import org.ow2.bonita.facade.def.majorElement.DataFieldDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ParticipantDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;


public class PackageFullDefinitionImpl extends PackageDefinitionImpl implements PackageFullDefinition {

  protected long dbid;
  protected Set<ParticipantDefinition> participants;
  protected Set<ApplicationDefinition> applications;
  protected Set<DataFieldDefinition> dataFields;
  protected Set<ProcessFullDefinition> processes;

  protected PackageFullDefinitionImpl() { }

  public PackageFullDefinitionImpl(PackageDefinitionUUID uuid) {
    super(uuid);
  }

  public PackageFullDefinitionImpl(PackageDefinitionUUID uuid, Date deployedDate, String deployedBy,
      String id, String name, String author, Set<String> classDependencies,
      String codepage, GraphConformance conformanceClass, String costUnit,
      String countrykey, String created, String description, String documentation,
      Set<ExternalPackageDefinition> externalPackages, Set<String> packageDependencies, String priorityUnit,
      PublicationStatus publicationStatus, Set<String> responsibles, String scriptType,
      String scriptVersion, String scriptGrammar,
      Set<TypeDeclarationDefinition> typeDeclarations, String vendor, String version,
      String xpdlVersion, BusinessArchive businessArchive,
      Set<ParticipantDefinition> participants, Set<ApplicationDefinition> applications,
      Set<DataFieldDefinition> dataFields, Set<ProcessFullDefinition> processes) {

    super(uuid, deployedDate, deployedBy, null, null, id, name,
        author, classDependencies, codepage, conformanceClass, costUnit,
        countrykey, created, description,documentation, externalPackages,
        packageDependencies, priorityUnit,
        publicationStatus, responsibles, scriptType, scriptVersion, scriptGrammar,
        typeDeclarations, vendor, version, xpdlVersion, businessArchive);

    this.participants = participants;
    this.applications = applications;
    this.dataFields = dataFields;
    this.processes = processes;
  }

  public PackageFullDefinitionImpl(PackageFullDefinition src) {
    super(src);
    Set<ParticipantDefinition> participants = src.getParticipants();
    if (participants != null) {
      this.participants = new HashSet<ParticipantDefinition>();
      for (ParticipantDefinition d : participants) {
        this.participants.add(new ParticipantDefinitionImpl(d));
      }
    }
    Set<ApplicationDefinition> applications = src.getApplications();
    if (applications != null) {
      this.applications = new HashSet<ApplicationDefinition>();
      for (ApplicationDefinition d : applications) {
        this.applications.add(new ApplicationDefinitionImpl(d));
      }
    }
    Set<DataFieldDefinition> dataFields = src.getDataFields();
    if (dataFields != null) {
      this.dataFields = new HashSet<DataFieldDefinition>();
      for (DataFieldDefinition d : dataFields) {
        this.dataFields.add(new DataFieldDefinitionImpl(d));
      }
    }
    Set<ProcessFullDefinition> processes = src.getProcesses();
    if (processes != null) {
      this.processes = new HashSet<ProcessFullDefinition>();
      for (ProcessFullDefinition d : processes) {
        this.processes.add(new ProcessFullDefinitionImpl(d));
      }
    }
  }

  public String toString() {
    String st = this.getClass().getName()
      + "[uuid: " + getUUID()
      + ", packageDefinitionUUID: " + getPackageDefinitionUUID()
      + ", packageId: " + getPackageId();
    if (processes != null) {
      for (ProcessFullDefinition process : processes) {
        st += ", process: " + process.toString();
      }
    }
    st +=  "]";
    return st;
  }

  public Set<ParticipantDefinition> getParticipants() {
    return participants;
  }


  public Set<ApplicationDefinition> getApplications() {
    return applications;
  }


  public Set<DataFieldDefinition> getDataFields() {
    return dataFields;
  }


  public Set<ProcessFullDefinition> getProcesses() {
    return processes;
  }

  public void setUndeployedDate(Date undeployedDate) {
    this.undeployedDate = undeployedDate;
  }
  public void setUndeployedBy(String undeployedBy) {
    this.undeployedBy = undeployedBy;
  }
  public boolean stateMatch(PackageState packageState) {
    return PackageState.DEPLOYED.equals(packageState) && getUndeployedBy() == null
      || PackageState.UNDEPLOYED.equals(packageState) && getUndeployedBy() != null;
  }
  /*
  public void end() {
    // nothing to do for the moment
  }
  */
}


